module Atmos
   class Response # :nodoc:
      attr_reader :http_response
      
      def initialize(response, action)
         @http_response = response
         @action = action

         if (response.kind_of?(Net::HTTPServerError))
            raise Atmos::Exceptions::ServerException, response.message
         elsif (response.kind_of?(Net::HTTPClientError))
            
            raise Atmos::Exceptions::AtmosException, "Atmos got a bad request.  This is probably a problem in this library." if (response.kind_of?(Net::HTTPBadRequest))
               
            Atmos::LOG.debug("#{response.class}")
            Atmos::LOG.debug("#{response.body}")
            
            Atmos::Parser::response_check_action_error(@action, response)
                        
         elsif (!response.kind_of?(Net::HTTPSuccess))
           raise Atmos::Exceptions::NotImplementedException, "This library doesn't handle these kinds of responses: #{response}"
         end

      end
      
      def each(header)
         if (!self[header].nil?)
            self[header].split(',').each do |elt|
               if (!elt.index('=').nil?)
                  key,val = elt.split('=')
                  yield key.strip, val.strip
               else
                  yield elt.strip
               end
            end
         end
      end
      
      def body
         @http_response.body
      end
      
      def method_missing(sym, *args)
         Atmos::LOG.info("method missing: #{sym}")
         header = "x-emc-#{sym.id2name.sub(/_/, '-')}"
         if (REST[@action][:return_headers].include?(header))
            Atmos::LOG.info("header: #{header}")
            rv = Atmos::Util.header2hash(header, @http_response[header])
            Atmos::LOG.debug("rv: #{rv.inspect}")
            return rv
         else
            return super.method_missing(sym, *args)
         end
      end
            
      def delta
         if (REST[@action][:return_headers].include?('x-emc-delta'))
            @http_response['x-emc-delta']
         else
            raise "A #{@action} request doesn't return a delta."
         end
      end
      
      def policy
         if (REST[@action][:return_headers].include?('x-emc-policy'))
            @http_response['x-emc-policy']
         else
            raise "A #{@action} request doesn't return a policy description."
         end
      end
            
      def id
         if (REST[@action][:return_headers].include?('location'))
            @http_response['location'][@http_response['location'].rindex('/')+1..-1]
         else
            raise "A #{@action} request doesn't return an id."
         end
      end
            
      def headers
         headers = {}
         @http_response.each do |header, value|
            headers[header] = value
         end
         headers
      end

      def [](header)
         @http_response[header]
      end

   end

   
         

   

#         while (response.kind_of?(Net::HTTPRedirection))
            
            #From http://railstips.org/blog/archives/2009/03/04/following-redirects-with-nethttp/
#            rurl = (response['location'].nil?) ? response.body.match(/<a href=\"([^>]+)\">/i)[1] : response['location']

#            puts("Got a redirect \nfrom: #{options[:url]}\n  to: #{rurl}")

#            if rurl.start_with?('/')
#            puts("Got a relative redirect url: #{options[:url]}")
#             options[:url] = URI.parse("#{url.scheme}://#{url.host}#{redirect_url}")
#           end

#            options[:redirects] = (options[:redirects].nil?) ? 0 : options[:redirects] += 1    
#            response = self.generic_request(options)
#         end
#         raise "Too many redirects (#{options[:redirects]}): #{url}" if (!options[:redirects].nil? && (options[:redirects] > 3))

end
