require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosMetadataTest < Test::Unit::TestCase
   
   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup
      @s = Atmos::Test::Util.static_store
     @obj = @s.create()
     @cleanup.push(@obj)
   end


   def teardown
         @cleanup.each do |obj|
            obj.delete
         end
      $stdout.flush
   end
   

   def test_default_metadata
      assert_equal({}, @obj.metadata)
      assert_equal({}, @obj.listable_metadata)
      assert_not_equal({}, @obj.system_metadata)
   end
   
   def test_add_metadata
      assert_equal({}, @obj.metadata)
      @obj.metadata['newmd'] = 'value'
      assert_equal({'newmd' => 'value'}, @obj.metadata)
      assert(@obj.metadata.has_key?('newmd'))
      assert_equal('value', @obj.metadata['newmd'])
   end
   
   def test_add_metadata_value_nil
      assert_equal({}, @obj.metadata)
      @obj.metadata['newmd'] = nil
      assert_equal({'newmd' => ""}, @obj.metadata)
      assert(@obj.metadata.has_key?('newmd'))
      assert_equal("", @obj.metadata['newmd'])
   end
   
   def test_add_metadata_value_boolean
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.metadata['newmd'] = true
      end
   end
   
   def test_add_metadata_value_symbol
      assert_raise Atmos::Exceptions::ArgumentException do
         @obj.metadata['newmd'] = :value
      end
   end
   
   def test_add_listable_metadata
      assert_equal({}, @obj.listable_metadata)
      @obj.listable_metadata['newmd'] = 'value'
      assert_equal({'newmd' => 'value'}, @obj.listable_metadata)
      assert(@obj.listable_metadata.has_key?('newmd'))
      assert_equal('value', @obj.listable_metadata['newmd'])
   end
   
   def test_modify_metadata
      assert_equal({}, @obj.metadata)
      @obj.metadata['newmd'] = 'value'
      assert_equal({'newmd' => 'value'}, @obj.metadata)

      @obj.metadata['newmd'] = 'newvalue'
      assert_equal({'newmd' => 'newvalue'}, @obj.metadata)
   end
   
   def test_modify_listable_metadata
      assert_equal({}, @obj.listable_metadata)
      @obj.listable_metadata['newmd'] = 'value'
      assert_equal({'newmd' => 'value'}, @obj.listable_metadata)

      @obj.listable_metadata['newmd'] = 'newvalue'
      assert_equal({'newmd' => 'newvalue'}, @obj.listable_metadata)
   end
   
   def test_delete_metadata
      assert_equal({}, @obj.metadata)
      @obj.metadata['fleur'] = 'dragan'
      @obj.metadata['john'] = 'kilroy'
      assert_equal({'fleur' => 'dragan', 'john' => 'kilroy'}, @obj.metadata)
      
      @obj.metadata.delete('john')
      assert_equal({'fleur' => 'dragan'}, @obj.metadata)

      @obj.metadata.delete('fleur')
      assert_equal({}, @obj.metadata)
   end
   
   def test_clear_metadata
      assert_equal({}, @obj.metadata)
      @obj.metadata['fleur'] = 'dragan'
      @obj.metadata['john'] = 'kilroy'
      assert_equal({'fleur' => 'dragan', 'john' => 'kilroy'}, @obj.metadata)
      
      @obj.metadata.clear
      assert_equal({}, @obj.metadata)
   end
   
   def test_delete_listable_metadata
      assert_equal({}, @obj.listable_metadata)
      @obj.listable_metadata['fleur'] = 'dragan'
      @obj.listable_metadata['john'] = 'kilroy'
      assert_equal({'fleur' => 'dragan', 'john' => 'kilroy'}, @obj.listable_metadata)
      
      @obj.listable_metadata.delete('john')
      assert_equal({'fleur' => 'dragan'}, @obj.listable_metadata)

      @obj.listable_metadata.delete('fleur')
      assert_equal({}, @obj.listable_metadata)
   end
   
   def test_clear_listable_metadata
      assert_equal({}, @obj.listable_metadata)
      @obj.listable_metadata['fleur'] = 'dragan'
      @obj.listable_metadata['john'] = 'kilroy'
      assert_equal({'fleur' => 'dragan', 'john' => 'kilroy'}, @obj.listable_metadata)
      
      @obj.listable_metadata.clear
      assert_equal({}, @obj.listable_metadata)
   end
   
   def test_get_system_metadata
      sys = @obj.system_metadata
      assert_not_nil(sys)
      assert(!sys.empty?)
   end
   
   def test_add_system_metadata
      assert_raise Atmos::Exceptions::NotImplementedException do
         @obj.system_metadata['api'] = 'ruby'
      end
   end
   
   def test_modify_system_metadata
      assert_raise Atmos::Exceptions::NotImplementedException do
         @obj.system_metadata['uid'] = 'foo'
      end
   end
   
   def test_delete_system_metadata
      assert_raise Atmos::Exceptions::NotImplementedException do
         @obj.system_metadata.delete('uid')
      end
   end
   
end




