require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosObjectGetTest < Test::Unit::TestCase

   def initialize(id)
      super(id)
      @cleanup = []

   end

   def setup
      @s = Atmos::Test::Util.static_store
   end


   def teardown
      @cleanup.each do |obj|
         obj.delete
      end
      $stdout.flush
   end

   def test_get_object_via_object_interface
      obj = @s.create
      @cleanup.push(obj)

      obj = @s.get(:id => obj.aoid)
      assert_not_nil(obj)
      assert_equal({}, obj.metadata)
      assert_equal({}, obj.listable_metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
   end
  
   def test_get_object_via_namespace_interface
      time = Time.now()
      time = time.to_i#{}"%.6f" % a.to_f   #=> "1195480202.282373"

      ns = "/foo/bar#{time}"
      obj = @s.create(:namespace => ns)
      obj = @s.get(:namespace => ns)
      
      assert_not_nil(obj)
      assert_equal({}, obj.metadata)
      assert_equal({}, obj.listable_metadata)
      assert_equal({Atmos::Test::Util.user => :full}, obj.user_acl)
      assert_equal({Atmos::Test::Util.group => :none}, obj.group_acl)
      
      obj.delete()
   end
  
end
