require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosObjectMiscTest < Test::Unit::TestCase

   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup
      @s   = Atmos::Test::Util.static_store
   end


   def teardown
      @cleanup.each do |obj|
         obj.delete
      end
   end


   def test_delete_existing_object
   end
  
   def test_exists_on_existant_object
      obj = @s.create()
      @cleanup.push(obj)
      assert(obj.exists?)
   end
   
   def test_exists_on_non_existant_object
      obj = @s.create()
      obj.delete
      assert(!obj.exists?)
   end
   
   def test_delete_nonexisting_object
      first = @s.create
      assert_not_nil(first)
      second = @s.get(:id => first.aoid)
      assert_not_nil(second)
      first.delete
      assert_raise Atmos::Exceptions::NoSuchObjectException do 
         second.delete
      end 
   end

   def test_truncate_text_object
      file = open(Atmos::Test::Util.small_text_filename)
      contents = file.read
      file.close
      data = open(Atmos::Test::Util.small_text_filename)
      obj = @s.create(:data => data)
      @cleanup.push(obj)
      assert_equal(contents, obj.data)
      
      obj.truncate
      assert_equal("", obj.data)
   end
   
   
   def test_truncate_binary_object
      file = open(Atmos::Test::Util.small_binary_filename)
      contents = file.read
      file.close
      data = open(Atmos::Test::Util.small_binary_filename)
      obj = @s.create(:data => data)
      @cleanup.push(obj)

#      assert_equal(contents, obj.data)
      
      obj.truncate
      assert_equal("", obj.data)
   end
   
   
end
