require 'credentials'
require 'test/unit'
require 'atmos'


class AtmosObjectReadTest < Test::Unit::TestCase

   def initialize(id)
      super(id)
      @cleanup = []
   end

   def setup    
      @s   = Atmos::Test::Util.static_store
      @md  = {'non' => 'listable', 'user' => 'metadata', 'a' => 'b'}
      @lmd = {'x' => 'a', 'y' => 'b', 'z' => 'c'}
      data = open(Atmos::Test::Util.small_binary_filename)
      obj  = @s.create(:metadata => @md, 
                       :listable_metadata => @lmd,
                       :data => File.new(Atmos::Test::Util.small_binary_filename, "rb"))
      @cleanup.push(obj)
      @aoid = obj.aoid
   end


  def teardown
    @cleanup.each do |obj|
      obj.delete
    end
  end
   
   
  def test_read_partial_data_as_stream
      obj = @s.get(:id => @aoid)
      
      file = open(Atmos::Test::Util.small_binary_filename)
      contents = file.read
      assert_equal(contents, obj.data)

      f = File.new(Atmos::Test::Util.small_binary_filename, "rb")
      contents = f.read(61)
      data = obj.data(0...60)
 
      readin = ""
      obj.data_as_stream(0...60) do |chunk|
         readin.concat(chunk)
      end
      
      assert_equal(61, readin.length)
      assert_equal(contents.length, readin.length)
      assert_equal(contents, readin)
   end

   def test_read_data_as_stream

      file = open(Atmos::Test::Util.small_binary_filename)
      contents = file.read
      obj = @s.get(:id => @aoid)
      obj.update(contents)
      assert_equal(contents, obj.data)

      readin = ""
      obj.data_as_stream do |chunk|
         readin.concat(chunk)
      end
      
      assert_equal(contents, readin)
   end

   def test_read_partial_data_as_string
      
      file = open(Atmos::Test::Util.small_binary_filename)
      contents = file.read
      obj = @s.get(:id => @aoid)
      obj.update(contents)
      assert_equal(contents, obj.data)


      f = File.new(Atmos::Test::Util.small_binary_filename, "rb")
      contents = f.read(61)
      data = obj.data(0...60)
      len = data.length
      assert_equal(61, obj.data(0...60).length)
      assert_equal(contents, obj.data(0...60))
   end

   def test_read_data_as_string
      file = open(Atmos::Test::Util.small_binary_filename)
      contents = file.read
      obj = @s.get(:id => @aoid)
      obj.update(contents)
      assert_equal(contents, obj.data)
      
      assert_equal(contents, obj.data)
      assert(obj.data.length > 0)
   end

end
