require 'test/unit'
require 'credentials'
require 'atmos'


class AtmosStoreConstructorTest < Test::Unit::TestCase

  def initialize( id )
    super(id)
  end

  def setup
  end


  def teardown
  end


  def test_connect_without_parser
     assert_raise Atmos::Exceptions::ArgumentException do
        Atmos::Store.new(:uri    => Atmos::Test::Util.url,
                         :uid    => Atmos::Test::Util.user,
                         :secret => Atmos::Test::Util.secret)
                      end
  end
  
  def test_connect_with_parser
     Atmos::Parser::parser = Atmos::Parser::NOKOGIRI
    s = Atmos::Store.new(:url    => Atmos::Test::Util.url,
                         :uid    => Atmos::Test::Util.uid,
                         :secret => Atmos::Test::Util.secret)
    assert_not_nil(s)
  end
  
  def test_redirect
     Atmos::Parser::parser = Atmos::Parser::NOKOGIRI
     if (!Atmos::Test::Util.redirect_url.nil?)
        assert_raise Atmos::Exceptions::NotImplementedException do
           Atmos::Store.new(:url    => Atmos::Test::Util.redirect_url,
                            :uid    => Atmos::Test::Util.uid,
                            :secret => Atmos::Test::Util.secret)
        end
     end
  end

  def test_bad_option
     Atmos::Parser::parser = Atmos::Parser::NOKOGIRI
     assert_raise Atmos::Exceptions::ArgumentException do
        Atmos::Store.new(:uri    => Atmos::Test::Util.url,
                         :uid    => Atmos::Test::Util.user,
                         :secret => Atmos::Test::Util.secret,
                         :foo    => true)
     end
  end

  def test_connect_to_bad_server_version
  end
  
end


class AtmosStoreTest < Test::Unit::TestCase

   def initialize( id )
      super(id)
      @cleanup = []
   end

   def setup
      @s = Atmos::Test::Util.static_store;
   end


   def teardown
         @cleanup.each do |obj|
            obj.delete
         end
   end
 
 
   def test_get_atmos_version
      rv = @s.server_version
      assert_not_nil(rv)
      assert_equal(String, rv.class)
      assert_equal(Atmos::Test::Util.version, rv) if (!Atmos::Test::Util.version.nil?)
   end
  
 
   def dont_test_iterate_through_listable_tags
      i = 0;
      ctag = "ruby-atmos-curseword#{$$}"
      obj = @s.create(:listable_metadata => {ctag => true})
      @cleanup.push(obj)
      obj.update("there's just too much confusion")
      assert_equal("there's just too much confusion", obj.data)
      
      @s.each_listable_tag do |tag|
         i += 1
      end
      
      assert(i > 0)
  end
  
  
   def test_iterate_through_listable_tags_with_token
   end
  
  
  
   def test_iterate_through_objects_with_tag
      srand()
      tag = "ruby-atmos-curseword-#{rand()}"
      
      for i in (1...10)
         obj = @s.create(:listable_metadata => {tag => true})
         @cleanup.push(obj)
         obj.update("there's just too much confusion")
         assert_equal("there's just too much confusion", obj.data)
      end
      
      i = 0
      @s.each_object_with_listable_tag(tag) do |obj|
         i += 1
         assert_equal("there's just too much confusion", obj.data)
         obj.update("i can't get no relief")
         assert_equal("i can't get no relief", obj.data)
      end
      assert_equal(9, i)
  end
  
end
